(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['jest-each', './'], factory);
  } else if (typeof exports !== "undefined") {
    factory(require('jest-each'), require('./'));
  } else {
    var mod = {
      exports: {}
    };
    factory(global.jestEach, global._);
    global.indexTest = mod.exports;
  }
})(this, function (_jestEach, _) {
  'use strict';

  var _jestEach2 = _interopRequireDefault(_jestEach);

  var _2 = _interopRequireDefault(_);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  describe('.addedDiff', function () {

    describe('base case', function () {
      describe('equal', function () {
        (0, _jestEach2.default)([['int', 1], ['string', 'a'], ['boolean', true], ['null', null], ['undefined', undefined], ['object', { a: 1 }], ['array', [1]], ['function', function () {
          return {};
        }], ['date', new Date()]]).test('returns empty object when given values of type %s are equal', function (type, value) {
          expect((0, _2.default)(value, value)).toEqual({});
        });
      });

      describe('not equal and not object', function () {
        (0, _jestEach2.default)([[1, 2], ['a', 'b'], [true, false], ['hello', null], ['hello', undefined], [null, undefined], [undefined, null], [null, { a: 1 }], ['872983', { areaCode: '+44', number: '872983' }], [100, function () {
          return {};
        }], [function () {
          return {};
        }, 100], [new Date('2017-01-01'), new Date('2017-01-02')]]).test('returns empty object when values are not equal (%s, %s)', function (lhs, rhs) {
          expect((0, _2.default)(lhs, rhs)).toEqual({});
        });
      });
    });

    describe('recursive case', function () {
      describe('object', function () {
        test('returns empty object when given objects are updated', function () {
          expect((0, _2.default)({ a: 1 }, { a: 2 })).toEqual({});
        });

        test('returns empty object when right hand side has deletion', function () {
          expect((0, _2.default)({ a: 1, b: 2 }, { a: 1 })).toEqual({});
        });

        test('returns subset of right hand side value when a key value has been added to the root', function () {
          expect((0, _2.default)({ a: 1 }, { a: 1, b: 2 })).toEqual({ b: 2 });
        });

        test('returns subset of right hand side value when a key value has been added deeply', function () {
          expect((0, _2.default)({ a: { b: 1 } }, { a: { b: 1, c: 2 } })).toEqual({ a: { c: 2 } });
        });

        test('returns subset of right hand side with added date', function () {
          expect((0, _2.default)({}, { date: new Date('2016') })).toEqual({ date: new Date('2016') });
        });
      });

      describe('arrays', function () {
        test('returns empty object when array is updated', function () {
          expect((0, _2.default)([1], [2])).toEqual({});
        });

        test('returns empty object when right hand side array has deletions', function () {
          expect((0, _2.default)([1, 2, 3], [1, 3])).toEqual({});
        });

        test('returns subset of right hand side array as object of indices to value when right hand side array has additions', function () {
          expect((0, _2.default)([1, 2, 3], [1, 2, 3, 9])).toEqual({ 3: 9 });
        });

        test('returns subset of right hand side with added date', function () {
          expect((0, _2.default)([], [new Date('2016')])).toEqual({ 0: new Date('2016') });
        });
      });

      describe('object create null', function () {
        test('returns subset of right hand side value when a key value has been added to the root', function () {
          var lhs = Object.create(null);
          var rhs = Object.create(null);
          lhs.a = 1;
          rhs.a = 1;
          rhs.b = 2;
          expect((0, _2.default)(lhs, rhs)).toEqual({ b: 2 });
        });

        test('returns subset of right hand side value when a key value has been added deeply', function () {
          var lhs = Object.create(null);
          var rhs = Object.create(null);
          lhs.a = { b: 1 };
          rhs.a = { b: 1, c: 2 };
          expect((0, _2.default)(lhs, rhs)).toEqual({ a: { c: 2 } });
        });

        test('returns subset of right hand side with added date', function () {
          var lhs = Object.create(null);
          var rhs = Object.create(null);
          rhs.date = new Date('2016');
          expect((0, _2.default)(lhs, rhs)).toEqual({ date: new Date('2016') });
        });
      });
    });
  });
});