(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['jest-each', './'], factory);
  } else if (typeof exports !== "undefined") {
    factory(require('jest-each'), require('./'));
  } else {
    var mod = {
      exports: {}
    };
    factory(global.jestEach, global._);
    global.indexTest = mod.exports;
  }
})(this, function (_jestEach, _) {
  'use strict';

  var _jestEach2 = _interopRequireDefault(_jestEach);

  var _2 = _interopRequireDefault(_);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  describe('.updatedDiff', function () {

    describe('base case', function () {
      describe('equal', function () {
        (0, _jestEach2.default)([['int', 1], ['string', 'a'], ['boolean', true], ['null', null], ['undefined', undefined], ['object', { a: 1 }], ['array', [1]], ['function', function () {
          return {};
        }], ['date', new Date()]]).test('returns empty object when given values of type %s are equal', function (type, value) {
          expect((0, _2.default)(value, value)).toEqual({});
        });
      });

      describe('not equal and not object', function () {
        (0, _jestEach2.default)([[1, 2], ['a', 'b'], [true, false], ['hello', null], ['hello', undefined], [null, undefined], [undefined, null], [null, { a: 1 }], ['872983', { areaCode: '+44', number: '872983' }], [100, function () {
          return {};
        }], [function () {
          return {};
        }, 100], [new Date('2017-01-01'), new Date('2017-01-02')]]).test('returns right hand side value when different to left hand side value (%s, %s)', function (lhs, rhs) {
          expect((0, _2.default)(lhs, rhs)).toEqual(rhs);
        });
      });
    });

    describe('recursive case', function () {
      describe('object', function () {
        test('returns right hand side value when given objects are different at root', function () {
          expect((0, _2.default)({ a: 1 }, { a: 2 })).toEqual({ a: 2 });
        });

        test('returns right hand side value when right hand side value is null', function () {
          expect((0, _2.default)({ a: 1 }, { a: null })).toEqual({ a: null });
        });

        test('returns subset of right hand side value when sibling objects differ', function () {
          expect((0, _2.default)({ a: { b: 1 }, c: 2 }, { a: { b: 1 }, c: 3 })).toEqual({ c: 3 });
        });

        test('returns subset of right hand side value when nested values differ', function () {
          expect((0, _2.default)({ a: { b: 1, c: 2 } }, { a: { b: 1, c: 3 } })).toEqual({ a: { c: 3 } });
        });

        test('returns subset of right hand side value when nested values differ at multiple paths', function () {
          expect((0, _2.default)({ a: { b: 1 }, c: 2, d: { e: 100 } }, { a: { b: 99 }, c: 3, d: { e: 100 } })).toEqual({ a: { b: 99 }, c: 3 });
        });

        test('returns empty object when deleted from right hand side', function () {
          expect((0, _2.default)({ a: 1, b: { c: 2 } }, { a: 1 })).toEqual({});
        });

        test('returns empty object when a key value has been added', function () {
          expect((0, _2.default)({ a: 1 }, { a: 1, b: 2 })).toEqual({});
        });

        test('returns subset of right hand side with updated date', function () {
          expect((0, _2.default)({ date: new Date('2016') }, { date: new Date('2017') })).toEqual({ date: new Date('2017') });
        });
      });

      describe('arrays', function () {
        test('returns right hand side value as object of indices to value when arrays are different', function () {
          expect((0, _2.default)([1], [2])).toEqual({ 0: 2 });
        });

        test('returns subset of right hand side array as object of indices to value when arrays differs at multiple indicies', function () {
          expect((0, _2.default)([1, 2, 3], [9, 8, 3])).toEqual({ 0: 9, 1: 8 });
        });

        test('returns subset of right hand side array as object of indices to value when right hand side array has deletions', function () {
          expect((0, _2.default)([1, 2, 3], [1, 3])).toEqual({ 1: 3 });
        });

        test('returns empty object when right hand side array has additions', function () {
          expect((0, _2.default)([1, 2, 3], [1, 2, 3, 9])).toEqual({});
        });

        test('returns subset of right hand side with updated date', function () {
          expect((0, _2.default)([new Date('2016')], [new Date('2017')])).toEqual({ 0: new Date('2017') });
        });
      });

      describe('date', function () {
        test('returns empty object when dates are equal', function () {
          expect((0, _2.default)(new Date('2016'), new Date('2016'))).toEqual({});
        });
      });

      describe('object create null', function () {
        test('returns right hand side value when given objects are different at root', function () {
          var lhs = Object.create(null);
          lhs.a = 1;
          var rhs = Object.create(null);
          rhs.a = 2;
          expect((0, _2.default)(lhs, rhs)).toEqual({ a: 2 });
        });

        test('returns subset of right hand side value when sibling objects differ', function () {
          var lhs = Object.create(null);
          lhs.a = { b: 1 };
          lhs.c = 2;
          var rhs = Object.create(null);
          rhs.a = { b: 1 };
          rhs.c = 3;
          expect((0, _2.default)(lhs, rhs)).toEqual({ c: 3 });
        });

        test('returns subset of right hand side value when nested values differ', function () {
          var lhs = Object.create(null);
          lhs.a = { b: 1, c: 2 };
          var rhs = Object.create(null);
          rhs.a = { b: 1, c: 3 };
          expect((0, _2.default)(lhs, rhs)).toEqual({ a: { c: 3 } });
        });

        test('returns subset of right hand side with updated date', function () {
          var lhs = Object.create(null);
          lhs.date = new Date('2016');
          var rhs = Object.create(null);
          rhs.date = new Date('2017');
          expect((0, _2.default)(lhs, rhs)).toEqual({ date: new Date('2017') });
        });
      });
    });
  });
});