(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['jest-each', './'], factory);
  } else if (typeof exports !== "undefined") {
    factory(require('jest-each'), require('./'));
  } else {
    var mod = {
      exports: {}
    };
    factory(global.jestEach, global._);
    global.indexTest = mod.exports;
  }
})(this, function (_jestEach, _) {
  'use strict';

  var _jestEach2 = _interopRequireDefault(_jestEach);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  describe('utils', function () {

    describe('.isDate', function () {
      (0, _jestEach2.default)([[new Date()], [new Date('2016')], [new Date('2016-01')], [new Date('2016-01-01')], [new Date('2016-01-01:14:45:20')], [new Date('Tue Feb 14 2017 14:45:20 GMT+0000 (GMT)')], [new Date('nonsense')]]).test('returns true when given a date object of %s', function (date) {
        expect((0, _.isDate)(date)).toBe(true);
      });

      (0, _jestEach2.default)([[100], ['100'], [false], [{ a: 100 }], [[100, 101, 102]], [Date.parse('2016')], [Date.now()]]).test('returns false when not given a date object of %s', function (x) {
        expect((0, _.isDate)(x)).toBe(false);
      });
    });

    describe('.isEmpty', function () {
      describe('returns true', function () {
        test('when given an empty object', function () {
          expect((0, _.isEmpty)({})).toBe(true);
        });

        test('when given an empty array', function () {
          expect((0, _.isEmpty)([])).toBe(true);
        });
      });

      describe('returns false', function () {
        test('when given an empty object', function () {
          expect((0, _.isEmpty)({ a: 1 })).toBe(false);
        });

        test('when given an empty array', function () {
          expect((0, _.isEmpty)([1])).toBe(false);
        });
      });
    });

    describe('.isObject', function () {
      test('returns true when value is an object', function () {
        expect((0, _.isObject)({})).toBe(true);
      });

      test('returns true when value is an array', function () {
        expect((0, _.isObject)([])).toBe(true);
      });

      (0, _jestEach2.default)([['int', 1], ['string', 'a'], ['boolean', true], ['null', null], ['undefined', undefined], ['function', function () {
        return {};
      }]]).test('returns false when value is of type: %s', function (type, value) {
        expect((0, _.isObject)(value)).toBe(false);
      });
    });

    describe('.properObject', function () {
      it('returns given object when object has keys and hasOwnProperty function', function () {
        var o = { a: 1 };
        var a = [1];
        expect((0, _.properObject)(o)).toBe(o);
        expect((0, _.properObject)(a)).toBe(a);
      });

      it('returns given value when value is not an object', function () {
        var o = 'hello';
        expect((0, _.properObject)(o)).toBe(o);
      });

      it('returns object that has given keys and hasOwnProperty function when given object is created from a null', function () {
        var o = Object.create(null);
        o.a = 1;
        var actual = (0, _.properObject)(o);
        expect(actual).toEqual({ a: 1 });
        expect(typeof actual.hasOwnProperty === 'function').toBe(true);
        expect(actual).not.toBe(o);
      });
    });
  });
});